#ifndef UTIL_H_
#define UTIL_H_

#include<fstream>
#include<stdexcept>
#include<cstdlib>

using namespace std;

class RuntimeException:public runtime_error {
public:
	RuntimeException(const std::string& err) : runtime_error(err) {}
};

// branch and bound for finding maximal cliques
int BBforMC(int* egs, int num, bool** interaction, int* tegs, int minD, int totalnum, ofstream& outfile, const char* terminalfile) {
	int i, j;
	int numEdges = 0;

	int minIndex = egs[0];
	int minIndexCnt = 0;
	int minDegree = 0;

	for(i = 0; i < num; i++) {
		if(interaction[egs[0]][egs[i]]) {
			minDegree++;
			numEdges++;
		}
	}
	// find the minimum degree element and count total number of edges of this graph
	int tmpDegree;
	int tmpIndexCnt=0;

	for(i = 1; i < num; i++) {
		tmpDegree = 0;
		tmpIndexCnt++;
		for(j = 0; j < num; j++) {
			if(interaction[egs[i]][egs[j]]) {
				tmpDegree++;
				numEdges++;
			}
		}
		if(minDegree>tmpDegree) {
			minDegree = tmpDegree;
			minIndex = egs[i];
			minIndexCnt = tmpIndexCnt;
		}
	}
	// if g is k-clique, we save it and exit subroutine, where k is no less than minD
	if(numEdges==num * (num - 1) && numEdges >= minD * (minD - 1)) {
		ifstream ionfile;
		ionfile.open(terminalfile, ios::in);
		int tmp;
		bool isgood = true;
		//int *tegs = NULL;
		int k = -1;
		int cnum = 0;
		int pcnum = 0;
		int bnum;
		int pnum;

		while(!ionfile.eof()) {
			isgood = true;
			ionfile >> tmp;
			if(tmp<0) {
				cnum = abs(tmp);
				if(pcnum>num&&pcnum<=totalnum) {
					bnum = 0;
					pnum = 0;
					for(i = 0; i < num; i++) {
						for(j = 0; j < pcnum; j++) {
							if(egs[i]==tegs[j]) {
								pnum++;
								break;
							}
						}
						if(pnum==bnum) {
							break;
						} else {
							bnum++;
						}
					}
					if(bnum==num&&bnum>0) {
						isgood = false;
						break;
					}
				}
				k = 0;
				pcnum = cnum;
			} else if(k>-1&&k<cnum) {
				tegs[k] = tmp;
				k++;
			}
		}
		if(pcnum>num&&isgood&&pcnum<=totalnum) {
			bnum = 0;
			pnum = 0;
			for(i = 0; i < num; i++) {
				for(j = 0; j < pcnum; j++) {
					if(egs[i]==tegs[j]) {
						pnum++;
						break;
					}
				}
				if(pnum==bnum) {
					break;
				} else {
					bnum++;
				}
			}
			if(bnum==num&&bnum>0) {
				isgood = false;
			}
		}		
	
		ionfile.clear();
		ionfile.close();

		if(isgood) {
			outfile << (-1 * num);
			outfile << " ";
			for(i = 0; i < num; i++) {
				outfile << egs[i] << " ";
			}
			outfile << endl;
			return 1;
		}
		delete []egs;
		return 0;
	} else if((numEdges < minD * (minD - 1)) || numEdges==0) {
		delete []egs;
		return 0;
	}

	//branch over here
	int *egsz = new int[minDegree + 1];
	egsz[0] = minIndex;
	j = 0;
	for(i = 0; i < num; i++) {
		if(interaction[minIndex][egs[i]]) {
			j++;
			egsz[j] = egs[i];
		}
	}

	int *tmpA = new int[num-1];
	for(i = 0; i < num-1; i++) {
		if(i < minIndexCnt)
			tmpA[i] = egs[i];
		else if(i >= minIndexCnt)
			tmpA[i] = egs[i+1];
	}

	delete []egs;

	int numfoundcliques = 0;
	numfoundcliques += BBforMC(egsz, minDegree + 1, interaction, tegs, minD, totalnum, outfile, terminalfile);
	//delete []egsz;
	numfoundcliques += BBforMC(tmpA, num-1, interaction, tegs, minD, totalnum, outfile, terminalfile);
	//delete []tmpA;
	return numfoundcliques;
}

#endif
